@@ -1,5 +1,4 @@
 <?php
-
 // This file is part of Moodle - http://moodle.org/
 //
 // Moodle is free software: you can redistribute it and/or modify
@@ -18,26 +17,28 @@
 /**
  * External user API
  *
- * @package    moodlecore
- * @subpackage user
- * @copyright  2009 Moodle Pty Ltd (http://moodle.com)
- * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
+ * @package   core_user
+ * @copyright 2009 Moodle Pty Ltd (http://moodle.com)
+ * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
 
 /**
  * Creates a user
  *
+ * @throws moodle_exception
  * @param stdClass $user user to create
  * @param bool $updatepassword if true, authentication plugin will update password.
+ * @param bool $triggerevent set false if user_created event should not be triggred.
+ *             This will not affect user_password_updated event triggering.
  * @return int id of the newly created user
  */
-function user_create_user($user, $updatepassword = true) {
-    global $DB;
+function user_create_user($user, $updatepassword = true, $triggerevent = true) {
+    global $CFG, $DB;
 
     // Set the timecreate field to the current time.
     if (!is_object($user)) {
-            $user = (object)$user;
+        $user = (object) $user;
     }
 
     // Check username.
@@ -61,6 +62,16 @@ function user_create_user($user, $updatepassword = true) {
         unset($user->password);
     }
 
+    // Make sure calendartype, if set, is valid.
+    if (!empty($user->calendartype)) {
+        $availablecalendartypes = \core_calendar\type_factory::get_list_of_calendar_types();
+        if (empty($availablecalendartypes[$user->calendartype])) {
+            $user->calendartype = $CFG->calendartype;
+        }
+    } else {
+        $user->calendartype = $CFG->calendartype;
+    }
+
     $user->timecreated = time();
     $user->timemodified = $user->timecreated;
 
@@ -78,14 +89,10 @@ function user_create_user($user, $updatepassword = true) {
         $authplugin->user_update_password($newuser, $userpassword);
     }
 
-    // Trigger event.
-    $event = \core\event\user_created::create(
-            array(
-                'objectid' => $newuserid,
-                'context' => $usercontext
-                )
-            );
-    $event->trigger();
+    // Trigger event If required.
+    if ($triggerevent) {
+        \core\event\user_created::create_from_userid($newuserid)->trigger();
+    }
 
     return $newuserid;
 }
@@ -93,18 +100,21 @@ function user_create_user($user, $updatepassword = true) {
 /**
  * Update a user with a user object (will compare against the ID)
  *
+ * @throws moodle_exception
  * @param stdClass $user the user to update
  * @param bool $updatepassword if true, authentication plugin will update password.
+ * @param bool $triggerevent set false if user_updated event should not be triggred.
+ *             This will not affect user_password_updated event triggering.
  */
-function user_update_user($user, $updatepassword = true) {
+function user_update_user($user, $updatepassword = true, $triggerevent = true) {
     global $DB;
 
-    // set the timecreate field to the current time
+    // Set the timecreate field to the current time.
     if (!is_object($user)) {
-            $user = (object)$user;
+        $user = (object) $user;
     }
 
-    //check username
+    // Check username.
     if (isset($user->username)) {
         if ($user->username !== core_text::strtolower($user->username)) {
             throw new moodle_exception('usernamelowercase');
@@ -118,7 +128,7 @@ function user_update_user($user, $updatepassword = true) {
     // Unset password here, for updating later, if password update is required.
     if ($updatepassword && isset($user->password)) {
 
-        //check password toward the password policy
+        // Check password toward the password policy.
         if (!check_password_policy($user->password, $errmsg)) {
             throw new moodle_exception($errmsg);
         }
@@ -127,6 +137,18 @@ function user_update_user($user, $updatepassword = true) {
         unset($user->password);
     }
 
+    // Make sure calendartype, if set, is valid.
+    if (!empty($user->calendartype)) {
+        $availablecalendartypes = \core_calendar\type_factory::get_list_of_calendar_types();
+        // If it doesn't exist, then unset this value, we do not want to update the user's value.
+        if (empty($availablecalendartypes[$user->calendartype])) {
+            unset($user->calendartype);
+        }
+    } else {
+        // Unset this variable, must be an empty string, which we do not want to update the calendartype to.
+        unset($user->calendartype);
+    }
+
     $user->timemodified = time();
     $DB->update_record('user', $user);
 
@@ -134,7 +156,7 @@ function user_update_user($user, $updatepassword = true) {
         // Get full user record.
         $updateduser = $DB->get_record('user', array('id' => $user->id));
 
-        // if password was set, then update its hash
+        // If password was set, then update its hash.
         if (isset($passwd)) {
             $authplugin = get_auth_plugin($updateduser->auth);
             if ($authplugin->can_change_password()) {
@@ -142,15 +164,10 @@ function user_update_user($user, $updatepassword = true) {
             }
         }
     }
-
-    // Trigger event.
-    $event = \core\event\user_updated::create(
-            array(
-                'objectid' => $user->id,
-                'context' => context_user::instance($user->id)
-                )
-            );
-    $event->trigger();
+    // Trigger event if required.
+    if ($triggerevent) {
+        \core\event\user_updated::create_from_userid($user->id)->trigger();
+    }
 }
 
 /**
@@ -167,8 +184,9 @@ function user_delete_user($user) {
 
 /**
  * Get users by id
- * @param array $userids id of users to retrieve
  *
+ * @param array $userids id of users to retrieve
+ * @return array
  */
 function user_get_users_by_id($userids) {
     global $DB;
@@ -194,22 +212,21 @@ function user_get_default_fields() {
 
 /**
  *
- * Give user record from mdl_user, build an array conntains
- * all user details
+ * Give user record from mdl_user, build an array contains all user details.
  *
  * Warning: description file urls are 'webservice/pluginfile.php' is use.
  *          it can be changed with $CFG->moodlewstextformatlinkstoimagesfile
  *
+ * @throws moodle_exception
  * @param stdClass $user user record from mdl_user
- * @param stdClass $context context object
  * @param stdClass $course moodle course
  * @param array $userfields required fields
  * @return array|null
  */
 function user_get_user_details($user, $course = null, array $userfields = array()) {
     global $USER, $DB, $CFG;
-    require_once($CFG->dirroot . "/user/profile/lib.php"); //custom field library
-    require_once($CFG->dirroot . "/lib/filelib.php");      // file handling on description and friends
+    require_once($CFG->dirroot . "/user/profile/lib.php"); // Custom field library.
+    require_once($CFG->dirroot . "/lib/filelib.php");      // File handling on description and friends.
 
     $defaultfields = user_get_default_fields();
 
@@ -223,8 +240,7 @@ function user_get_user_details($user, $course = null, array $userfields = array(
         }
     }
 
-
-    // Make sure id and fullname are included
+    // Make sure id and fullname are included.
     if (!in_array('id', $userfields)) {
         $userfields[] = 'id';
     }
@@ -259,7 +275,7 @@ function user_get_user_details($user, $course = null, array $userfields = array(
     } else {
         $canviewuseremail = false;
     }
-    $cannotviewdescription   = !empty($CFG->profilesforenrolledusersonly) && !$currentuser && !$DB->record_exists('role_assignments', array('userid'=>$user->id));
+    $cannotviewdescription   = !empty($CFG->profilesforenrolledusersonly) && !$currentuser && !$DB->record_exists('role_assignments', array('userid' => $user->id));
     if (!empty($course)) {
         $canaccessallgroups = has_capability('moodle/site:accessallgroups', $context);
     } else {
@@ -267,7 +283,7 @@ function user_get_user_details($user, $course = null, array $userfields = array(
     }
 
     if (!$currentuser && !$canviewdetailscap && !has_coursecontact_role($user->id)) {
-        // skip this user details
+        // Skip this user details.
         return null;
     }
 
@@ -305,28 +321,27 @@ function user_get_user_details($user, $course = null, array $userfields = array(
             }
         }
         $fields->close();
-        // unset customfields if it's empty
+        // Unset customfields if it's empty.
         if (empty($userdetails['customfields'])) {
             unset($userdetails['customfields']);
         }
     }
 
-    // profile image
+    // Profile image.
     if (in_array('profileimageurl', $userfields)) {
-        $profileimageurl = moodle_url::make_pluginfile_url($usercontext->id, 'user', 'icon', NULL, '/', 'f1');
+        $profileimageurl = moodle_url::make_pluginfile_url($usercontext->id, 'user', 'icon', null, '/', 'f1');
         $userdetails['profileimageurl'] = $profileimageurl->out(false);
     }
     if (in_array('profileimageurlsmall', $userfields)) {
-        $profileimageurlsmall = moodle_url::make_pluginfile_url($usercontext->id, 'user', 'icon', NULL, '/', 'f2');
+        $profileimageurlsmall = moodle_url::make_pluginfile_url($usercontext->id, 'user', 'icon', null, '/', 'f2');
         $userdetails['profileimageurlsmall'] = $profileimageurlsmall->out(false);
     }
 
-    //hidden user field
+    // Hidden user field.
     if ($canviewhiddenuserfields) {
         $hiddenfields = array();
-        // address, phone1 and phone2 not appears in hidden fields list
-        // but require viewhiddenfields capability
-        // according to user/profile.php
+        // Address, phone1 and phone2 not appears in hidden fields list but require viewhiddenfields capability
+        // according to user/profile.php.
         if ($user->address && in_array('address', $userfields)) {
             $userdetails['address'] = $user->address;
         }
@@ -402,9 +417,9 @@ function user_get_user_details($user, $course = null, array $userfields = array(
         }
     }
 
-    if (in_array('email', $userfields) && ($isadmin // The admin is allowed the users email
-      or $currentuser // Of course the current user is as well
-      or $canviewuseremail  // this is a capability in course context, it will be false in usercontext
+    if (in_array('email', $userfields) && ($isadmin // The admin is allowed the users email.
+      or $currentuser // Of course the current user is as well.
+      or $canviewuseremail  // This is a capability in course context, it will be false in usercontext.
       or in_array('email', $showuseridentityfields)
       or $user->maildisplay == 1
       or ($user->maildisplay == 2 and enrol_sharing_course($user, $USER)))) {
@@ -418,7 +433,7 @@ function user_get_user_details($user, $course = null, array $userfields = array(
         }
     }
 
-    //Departement/Institution/Idnumber are not displayed on any profile, however you can get them from editing profile.
+    // Departement/Institution/Idnumber are not displayed on any profile, however you can get them from editing profile.
     if ($isadmin or $currentuser or in_array('idnumber', $showuseridentityfields)) {
         if (in_array('idnumber', $userfields) && $user->idnumber) {
             $userdetails['idnumber'] = $user->idnumber;
@@ -430,13 +445,13 @@ function user_get_user_details($user, $course = null, array $userfields = array(
         }
     }
     if ($isadmin or $currentuser or in_array('department', $showuseridentityfields)) {
-        if (in_array('department', $userfields) && isset($user->department)) { //isset because it's ok to have department 0
+        if (in_array('department', $userfields) && isset($user->department)) { // Isset because it's ok to have department 0.
             $userdetails['department'] = $user->department;
         }
     }
 
     if (in_array('roles', $userfields) && !empty($course)) {
-        // not a big secret
+        // Not a big secret.
         $roles = get_user_roles($context, $user->id, false);
         $userdetails['roles'] = array();
         foreach ($roles as $role) {
@@ -449,7 +464,7 @@ function user_get_user_details($user, $course = null, array $userfields = array(
         }
     }
 
-    // If groups are in use and enforced throughout the course, then make sure we can meet in at least one course level group
+    // If groups are in use and enforced throughout the course, then make sure we can meet in at least one course level group.
     if (in_array('groups', $userfields) && !empty($course) && $canaccessallgroups) {
         $usergroups = groups_get_all_groups($course->id, $user->id, $course->defaultgroupingid,
                 'g.id, g.name,g.description,g.descriptionformat');
@@ -458,11 +473,11 @@ function user_get_user_details($user, $course = null, array $userfields = array(
             list($group->description, $group->descriptionformat) =
                 external_format_text($group->description, $group->descriptionformat,
                         $context->id, 'group', 'description', $group->id);
-            $userdetails['groups'][] = array('id'=>$group->id, 'name'=>$group->name,
-                'description'=>$group->description, 'descriptionformat'=>$group->descriptionformat);
+            $userdetails['groups'][] = array('id' => $group->id, 'name' => $group->name,
+                'description' => $group->description, 'descriptionformat' => $group->descriptionformat);
         }
     }
-    //list of courses where the user is enrolled
+    // List of courses where the user is enrolled.
     if (in_array('enrolledcourses', $userfields) && !isset($hiddenfields['mycourses'])) {
         $enrolledcourses = array();
         if ($mycourses = enrol_get_users_courses($user->id, true)) {
@@ -480,14 +495,14 @@ function user_get_user_details($user, $course = null, array $userfields = array(
         }
     }
 
-    //user preferences
+    // User preferences.
     if (in_array('preferences', $userfields) && $currentuser) {
         $preferences = array();
         $userpreferences = get_user_preferences();
-         foreach($userpreferences as $prefname => $prefvalue) {
+        foreach ($userpreferences as $prefname => $prefvalue) {
             $preferences[] = array('name' => $prefname, 'value' => $prefvalue);
-         }
-         $userdetails['preferences'] = $preferences;
+        }
+        $userdetails['preferences'] = $preferences;
     }
 
     return $userdetails;
@@ -497,14 +512,14 @@ function user_get_user_details($user, $course = null, array $userfields = array(
  * Tries to obtain user details, either recurring directly to the user's system profile
  * or through one of the user's course enrollments (course profile).
  *
- * @param object $user The user.
+ * @param stdClass $user The user.
  * @return array if unsuccessful or the allowed user details.
  */
 function user_get_user_details_courses($user) {
     global $USER;
     $userdetails = null;
 
-    //  Get the courses that the user is enrolled in (only active).
+    // Get the courses that the user is enrolled in (only active).
     $courses = enrol_get_users_courses($user->id, true);
 
     $systemprofile = false;
@@ -518,7 +533,7 @@ function user_get_user_details_courses($user) {
     } else {
         // Try through course profile.
         foreach ($courses as $course) {
-            if ($can_view_user_details_cap($user, $course) || ($user->id == $USER->id) || has_coursecontact_role($user->id)) {
+            if (can_view_user_details_cap($user, $course) || ($user->id == $USER->id) || has_coursecontact_role($user->id)) {
                 $userdetails = user_get_user_details($user, $course);
             }
         }
@@ -530,8 +545,8 @@ function user_get_user_details_courses($user) {
 /**
  * Check if $USER have the necessary capabilities to obtain user details.
  *
- * @param object $user
- * @param object $course if null then only consider system profile otherwise also consider the course's profile.
+ * @param stdClass $user
+ * @param stdClass $course if null then only consider system profile otherwise also consider the course's profile.
  * @return bool true if $USER can view user details.
  */
 function can_view_user_details_cap($user, $course = null) {
@@ -551,7 +566,34 @@ function can_view_user_details_cap($user, $course = null) {
  * @param string $pagetype current page type
  * @param stdClass $parentcontext Block's parent context
  * @param stdClass $currentcontext Current context of block
+ * @return array
  */
 function user_page_type_list($pagetype, $parentcontext, $currentcontext) {
-    return array('user-profile'=>get_string('page-user-profile', 'pagetype'));
+    return array('user-profile' => get_string('page-user-profile', 'pagetype'));
 }
+
+/**
+ * Count the number of failed login attempts for the given user, since last successful login.
+ *
+ * @param int|stdclass $user user id or object.
+ * @param bool $reset Resets failed login count, if set to true.
+ *
+ * @return int number of failed login attempts since the last successful login.
+ */
+function user_count_login_failures($user, $reset = true) {
+    global $DB;
+
+    if (!is_object($user)) {
+        $user = $DB->get_record('user', array('id' => $user), '*', MUST_EXIST);
+    }
+    if ($user->deleted) {
+        // Deleted user, nothing to do.
+        return 0;
+    }
+    $count = get_user_preferences('login_failed_count_since_success', 0, $user);
+    if ($reset) {
+        set_user_preference('login_failed_count_since_success', 0, $user);
+    }
+    return $count;
+}
+
